/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import { unreachableCase } from "@fluidframework/core-utils/internal";
import { replaceAtomRevisions } from "../../core/index.js";
import { MarkListFactory } from "./markListFactory.js";
import { NoopMarkType } from "./types.js";
export function replaceRevisions(changeset, revisionsToReplace, newRevision) {
    const updatedMarks = new MarkListFactory();
    for (const mark of changeset) {
        const updatedMark = updateMark(mark, revisionsToReplace, newRevision);
        updatedMarks.push(updatedMark);
    }
    return updatedMarks.list;
}
function updateMark(mark, revisionsToReplace, newRevision) {
    const updatedMark = { ...updateEffect(mark, revisionsToReplace, newRevision) };
    if (mark.cellId !== undefined) {
        updatedMark.cellId = replaceAtomRevisions(mark.cellId, revisionsToReplace, newRevision);
    }
    if (mark.changes !== undefined) {
        updatedMark.changes = replaceAtomRevisions(mark.changes, revisionsToReplace, newRevision);
    }
    return updatedMark;
}
function updateEffect(mark, revisionsToReplace, newRevision) {
    const type = mark.type;
    switch (type) {
        case NoopMarkType:
            return mark;
        case "AttachAndDetach":
            return {
                ...mark,
                attach: updateEffect(mark.attach, revisionsToReplace, newRevision),
                detach: updateEffect(mark.detach, revisionsToReplace, newRevision),
            };
        case "Insert":
        case "MoveIn":
        case "MoveOut":
        case "Remove":
            return updateEffectRevision(mark, revisionsToReplace, newRevision);
        default:
            unreachableCase(type);
    }
}
function updateEffectRevision(effect, revisionsToReplace, newRevision) {
    return revisionsToReplace.has(effect.revision)
        ? effectWithRevision(effect, newRevision)
        : effect;
}
function effectWithRevision(effect, revision) {
    const updated = { ...effect, revision };
    if (revision === undefined) {
        delete updated.revision;
    }
    return updated;
}
//# sourceMappingURL=replaceRevisions.js.map